;nyquist plug-in
;version 4
;type analyze
;name "Measure SPL"
;action "Measuring SPL level..."
;manpage https:/www.conradhoffman.com//Audacity_SPL.htm
;author "Conrad Hoffman and Steve Daulton based on Measure RMS"
;copyright "Released under terms of the GNU General Public License version 2"
;release 1.1
;debugbutton false

;; Plugin adds a constant to the dB measurement to convert it to SPL (sound pressure level).

;; You will have to determine constants for your mics and enter them when you run
;; the plug-in. They will be remembered until you change them again. See manpage for
;; complete details.

;; Comment out the next three lines with an extra semicolon if hardcoding values:

;control leftoffset "Left channel calibration constant" float-text "dB" 104.93 0 nil
;control rightoffset "Right channel calibration constant" float-text "dB" 144.8049 0 nil
;control monooffset "Mono calibration constant" float-text "dB" 104.93 0 nil

(defmacro set-default (symb val)
  ;;; Set symbol to val if symbol not bound.
  `(unless (boundp ',symb)
    (setf ,symb ,val)))

;;; Hardcoded values used when ;control values not used- change these to your values.
(set-default leftoffset 104.93)
(set-default rightoffset 144.8049)
(set-default monooffset 104.93)

(defun get-suffix (offset)
  (if (= offset 0) "dB" "dB SPL"))

(setf *float-format* "%.2f")  ;2 decimal places, which is one more than you really need

(let ((rms (get '*selection* 'rms)))
  (if (arrayp rms)
      (format nil "Left: \t~a ~a~%Right: \t~a ~a"
              (+ leftoffset (linear-to-db (aref rms 0)))
              (get-suffix leftoffset)
              (+ rightoffset (linear-to-db (aref rms 1)))
              (get-suffix rightoffset))
      (format nil "Mono: ~a ~a"
              (+ monooffset (linear-to-db rms))
              (get-suffix monooffset))))
